@@ -2,45 +2,85 @@
 
 use std::marker::PhantomData;
 
-#[cfg(test)]
-mod test;
-
 /// Trait describing frame rates.
 pub trait FrameRate {
-    fn as_tuple() -> (u16, u16);
+    const DROP_FRAME: bool;
+    const FPS_FLOAT: f32;
+    const FPS_INT: u32;
+
+    fn calculate_frame_number(
+        hour: u32,
+        minute: u32,
+        second: u32,
+        frame: u32,
+    ) -> Option<u32> {
+        let frames_per_minute = Self::FPS_INT * 60;
+        let frames_per_hour = frames_per_minute * 60;
+
+        let mut frame_number = (frames_per_hour * hour)
+            + (frames_per_minute * minute)
+            + (Self::FPS_INT * second) + frame;
+
+        if Self::DROP_FRAME {
+            let minutes = (60 * hour) + minute;
+            let drop_frames_per_minute =
+                (Self::FPS_FLOAT * (6.0 / 100.0)).round() as u32;
+            frame_number -= drop_frames_per_minute * (minutes - (minutes / 10));
+        }
+
+        Some(frame_number)
+    }
 }
 
 macro_rules! create_frame_rate {
-    ($frame_rate_name:ident, $numerator:expr, $denominator:expr) => (
+    ($frame_rate_name:ident,
+     $frame_rate_int:expr,
+     $frame_rate_float:expr,
+     $drop_frame:expr) => (
         #[derive(Debug, PartialEq)]
         pub struct $frame_rate_name;
 
         impl FrameRate for $frame_rate_name {
-            fn as_tuple() -> (u16, u16) { ($numerator, $denominator) }
+            const DROP_FRAME: bool = $drop_frame;
+            const FPS_FLOAT: f32 = $frame_rate_float as f32;
+            const FPS_INT: u32 = $frame_rate_int;
         }
     )
 }
 
-create_frame_rate!(FrameRate24, 24, 1);
-create_frame_rate!(FrameRate25, 25, 1);
-create_frame_rate!(FrameRate30, 30, 1);
-create_frame_rate!(FrameRate50, 50, 1);
-create_frame_rate!(FrameRate60, 60, 1);
-create_frame_rate!(FrameRate2398, 24000, 1001);
-create_frame_rate!(FrameRate2997, 30000, 1001);
-create_frame_rate!(FrameRate5994, 60000, 1001);
+create_frame_rate!(FrameRate24, 24, 24, false);
+create_frame_rate!(FrameRate25, 25, 25, false);
+create_frame_rate!(FrameRate30, 30, 30, false);
+create_frame_rate!(FrameRate50, 50, 50, false);
+create_frame_rate!(FrameRate60, 60, 60, false);
+create_frame_rate!(
+    FrameRate2398,
+    24,
+    23.97602397602397602397602397602397602397602397602397602397602397602,
+    true
+);
+create_frame_rate!(
+    FrameRate2997,
+    30,
+    29.97002997002997002997002997002997002997002997002997002997002997002,
+    true
+);
+create_frame_rate!(
+    FrameRate5994,
+    60,
+    59.94005994005994005994005994005994005994005994005994005994005994005,
+    true
+);
 
 /// Representation of a timecode
 #[derive(Debug, PartialEq)]
 pub struct Timecode<FrameRate> {
+    pub frame_number: u32,
+    pub drop_frame: bool,
     pub hour: u8,
     pub minute: u8,
     pub second: u8,
     pub frame: u8,
-    frame_rate_numerator: u16,
-    frame_rate_denominator: u16,
-    pub drop_frame: bool,
-    pub frame_number: u32,
     frame_rate: PhantomData<FrameRate>,
 }
 
@@ -64,10 +104,7 @@ impl<T> Timecode<T> {
     ///     _ => panic!()
     /// };
     ///
-    /// assert_eq!(timecode.hour, 10);
-    /// assert_eq!(timecode.minute, 0);
-    /// assert_eq!(timecode.second, 0);
-    /// assert_eq!(timecode.frame, 0);
+    /// assert_eq!(timecode.frame_number, 864000);
     /// ```
     pub fn new(
         hour: u8,
@@ -88,58 +125,24 @@ impl<T> Timecode<T> {
             return Err("Invalid second");
         }
 
-        let (frame_rate_numerator, frame_rate_denominator) = T::as_tuple();
-
-        let frame_number = match get_frame_count(
+        let frame_number = match T::calculate_frame_number(
             hour as u32,
             minute as u32,
             second as u32,
             frame as u32,
-            frame_rate_numerator,
-            frame_rate_denominator,
         ) {
             Some(frame_number) => frame_number,
             None => return Err("Invalid frame count"),
         };
 
-        let drop_frame = match frame_rate_denominator {
-            1 => false,
-            1001 => true,
-            _ => panic!(),
-        };
-
         Ok(Timecode {
+            frame_number,
             hour,
             minute,
             second,
             frame,
-            frame_rate_numerator,
-            frame_rate_denominator,
-            drop_frame,
-            frame_number,
+            drop_frame: T::DROP_FRAME,
             frame_rate: PhantomData,
         })
     }
 }
-
-fn get_frame_count(
-    hour: u32,
-    minute: u32,
-    second: u32,
-    frame: u32,
-    frame_rate_numerator: u16,
-    frame_rate_denominator: u16,
-) -> Option<u32> {
-    let fps: f32 = frame_rate_numerator as f32 / frame_rate_denominator as f32;
-    let drop_frames_per_minute = (fps * (6.0 / 100.0)).round() as u32;
-    let fps_rounded = fps.round() as u32;
-    let frames_per_minute = fps_rounded * 60;
-    let frames_per_hour = frames_per_minute * 60;
-    let minutes = (60 * hour) + minute;
-
-    let frame_number = ((frames_per_hour * hour) + (frames_per_minute * minute)
-        + (fps_rounded * second) + frame)
-        - (drop_frames_per_minute * (minutes - (minutes / 10)));
-
-    Some(frame_number)
-}
